import {useEffect} from "react";
import PropTypes from "prop-types";
import {useDispatch,useSelector} from "react-redux";
import {I18n} from "react-polyglot";
import {enGB,es} from "date-fns/locale"
import {LocalizationProvider} from "@mui/x-date-pickers";
import {AdapterDateFns} from "@mui/x-date-pickers/AdapterDateFns";
import {cambiarIdioma} from "../redux";
import saasMessages from "../i18n";

const supportedLocales={en: enGB,es};

const IdiomaInner=({messages,children}) => {
	const idioma=useSelector(store => store.ui.idioma);

	return (
		<I18n locale={idioma} messages={messages[idioma]}>
			<LocalizationProvider dateAdapter={AdapterDateFns} adapterLocale={supportedLocales[idioma]}>
				{children}
			</LocalizationProvider>
		</I18n>
	);
};

IdiomaInner.propTypes={
	messages: PropTypes.object,
	children: PropTypes.element.isRequired
};

const Idioma=({messages,idioma,children}) => {
	const dispatch=useDispatch();

	useEffect(() => {
		if(idioma){
			dispatch(cambiarIdioma(idioma));
		}
	},[dispatch,idioma]);

	const m={};
	Object.keys(messages).forEach(key => {
		m[key]={
			...messages[key],
			saas: saasMessages[key]
		};
	});

	return (
		<IdiomaInner messages={m}>
			{children}
		</IdiomaInner>
	);
};

Idioma.propTypes={
	messages: PropTypes.object,
	idioma: PropTypes.string,
	children: PropTypes.element.isRequired
};

export default Idioma;