import {createSlice,createAsyncThunk} from "@reduxjs/toolkit";
import {loginApi} from "../api/loginApi";
import jwtDecode from "jwt-decode";

const loginSlice = createSlice({
	name: "login",
	initialState: {
		cargando: false,
		autenticado: false,
		token: null,
		refreshToken: null,
		expiracion: null,
		usuario: null,
		perfiles: [],
		error: null
	},
	reducers: {
		logout: state => {
			state.cargando = false;
			state.autenticado = false;
			state.token = null;
			state.refreshToken = null;
			state.expiracion = null;
			state.usuario = null;
			state.perfiles = [];
			state.error = null;
		}
	},
	extraReducers(builder) {
		builder
			.addCase(requestToken.pending,state => {
				state.cargando = true;
				state.error = null;
			})
			.addCase(requestToken.fulfilled,(state,action) => {
				const token = action.payload.access_token;
				const jwtToken = jwtDecode(token);
				const expiracion = new Date();
				expiracion.setSeconds(expiracion.getSeconds() + action.payload.expires_in);
				state.cargando = false;
				state.autenticado = true;
				state.token = token;
				state.refreshToken = action.payload.refresh_token;
				state.expiracion = expiracion.getTime();
				state.usuario = jwtToken.name;
				state.perfiles = jwtToken.authorities;
			})
			.addCase(requestToken.rejected,(state,action) => {
				console.log(action);
				state.cargando = false;
				state.autenticado = false;
				state.token = null;
				state.refreshToken = null;
				state.expiracion = null;
				state.usuario = null;
				state.perfiles = [];
				state.error = action.error?.message;
			})
			.addCase(refreshToken.pending,state => {
				state.cargando = true;
			})
			.addCase(refreshToken.fulfilled,(state,action) => {
				const expiracion = new Date();
				expiracion.setSeconds(expiracion.getSeconds() + action.payload.expires_in);
				state.token = action.payload.access_token;
				state.refreshToken = action.payload.refresh_token;
				state.expiracion = expiracion.getTime();
			})
			.addCase(refreshToken.rejected,(state,action) => {
				state.cargando = false;
				state.autenticado = false;
				state.token = null;
				state.refreshToken = null;
				state.expiracion = null;
				state.usuario = null;
				state.perfiles = [];
				state.error = action.error?.message;
			});
	}
});

export const requestToken = createAsyncThunk("login/requestToken",async (payload) => {
	const response = await loginApi(payload.devURL).login(payload.clientCredentials,payload.data);
	return response.data;
});

export const refreshToken = createAsyncThunk("login/refreshToken",async (devURL,clientCredentials,refreshToken) => {
	const response = await loginApi(devURL).refreshToken(clientCredentials,refreshToken);
	return response.data;
});

export const {logout} = loginSlice.actions;

export default loginSlice.reducer;